@@ -36,6 +36,10 @@
 #include "subprocs.h"
 #include "gs-debug.h"
 
+#ifdef HAVE_SHAPE_EXT
+#include <X11/extensions/shape.h>
+#endif
+
 static void gs_window_class_init (GSWindowClass *klass);
 static void gs_window_init       (GSWindow      *window);
 static void gs_window_finalize   (GObject       *object);
@@ -93,6 +97,10 @@ struct GSWindowPrivate
         gdouble    last_y;
 
         GTimer    *timer;
+
+#ifdef HAVE_SHAPE_EXT
+        int        shape_event_base;
+#endif
 };
 
 enum {
@@ -610,6 +618,17 @@ x11_window_is_ours (Window window)
         return ret;
 }
 
+#ifdef HAVE_SHAPE_EXT
+static void
+unshape_window (GSWindow *window)
+{
+        gdk_window_shape_combine_region (GTK_WIDGET (window)->window,
+                                         NULL,
+                                         0,
+                                         0);
+}
+#endif
+
 static void
 gs_window_xevent (GSWindow  *window,
                   GdkXEvent *xevent)
@@ -646,6 +665,15 @@ gs_window_xevent (GSWindow  *window,
                         break;
                 }
         default:
+                /* extension events */
+#ifdef HAVE_SHAPE_EXT
+                if (ev->xany.type == (window->priv->shape_event_base + ShapeNotify)) {
+                        /*XShapeEvent *xse = (XShapeEvent *) ev;*/
+                        unshape_window (window);
+                        gs_debug ("Window was reshaped!");
+                }
+#endif
+
                 break;
         }
 
@@ -678,6 +706,25 @@ select_popup_events (void)
         gdk_error_trap_pop ();
 }
 
+static void
+window_select_shape_events (GSWindow *window)
+{
+#ifdef HAVE_SHAPE_EXT
+        unsigned long events;
+        int           shape_error_base;
+
+        gdk_error_trap_push ();
+
+        if (XShapeQueryExtension (GDK_DISPLAY (), &window->priv->shape_event_base, &shape_error_base)) {
+                events = ShapeNotifyMask;
+                XShapeSelectInput (GDK_DISPLAY (), GDK_WINDOW_XID (GTK_WIDGET (window)->window), events);
+        }
+
+        gdk_display_sync (gdk_display_get_default ());
+        gdk_error_trap_pop ();
+#endif
+}
+
 static void
 gs_window_real_show (GtkWidget *widget)
 {
@@ -701,6 +748,7 @@ gs_window_real_show (GtkWidget *widget)
         add_watchdog_timer (window, 30000);
 
         select_popup_events ();
+        window_select_shape_events (window);
         gdk_window_add_filter (NULL, (GdkFilterFunc)xevent_filter, window);
 }
 
@@ -997,7 +1045,7 @@ forward_key_events (GSWindow *window)
 {
         window->priv->key_events = g_list_reverse (window->priv->key_events);
 
-        while (window->priv->key_events) {
+        while (window->priv->key_events != NULL) {
                 GdkEventKey *event = window->priv->key_events->data;
 
                 gtk_window_propagate_key_event (GTK_WINDOW (window), event);
@@ -1468,6 +1516,24 @@ gs_window_request_unlock (GSWindow *window)
         g_signal_emit (window, signals [DIALOG_UP], 0);
 }
 
+void
+gs_window_cancel_unlock_request (GSWindow  *window)
+{
+        /* FIXME: This is a bit of a hammer approach...
+	 * Maybe we should send a delete-event to
+	 * the plug?
+	 */
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+	if (window->priv->lock_socket == NULL) {
+		return;
+        }
+
+	if (window->priv->lock_pid > 0) {
+		kill (window->priv->lock_pid, SIGTERM);
+        }
+}
+
 void
 gs_window_set_lock_enabled (GSWindow *window,
                             gboolean  lock_enabled)
